
'use strict';

goog.provide('Blockly.Luge.Script');

goog.require('Blockly.Luge');

Blockly.Luge['procedure'] = function(block) {
  var text_proc_name = block.getFieldValue('PROC_NAME');
  var text_proc_cat = block.getFieldValue('PROC_CAT');
  var statements_proc = Blockly.Luge.statementToCode(block, 'PROC');
  
  var code = '// ' + text_proc_name + '\n' + 
  'Debut ( "' + text_proc_name + '" , "' + text_proc_cat + ' )\n'+
  statements_proc + '\n' + 
  'Fin\n';
  return code;
};

Blockly.Luge['wait'] = function(block) {
  var value_duration = Blockly.Luge.valueToCode(block, 'DURATION', Blockly.Luge.ORDER_ATOMIC);
  var dropdown_unit = block.getFieldValue('UNIT');  
  
  var code = 'Attente ( ' + value_duration + ')\n';
  return code;
};

Blockly.Luge['for_loop'] = function(block) {
	
  var value_iter = Blockly.Luge.valueToCode(block, 'ITER', Blockly.Luge.ORDER_ATOMIC);
  var statements_code = Blockly.Luge.statementToCode(block, 'CODE');
  
  var code = 'Pour ( ' + value_iter + ' )\n'+
  statements_code +
  'FinPour\n';
  return code;
};

Blockly.Luge['while_loop'] = function(block) {
	
  var value_var1 = Blockly.Luge.valueToCode(block, 'VAR1', Blockly.Luge.ORDER_ATOMIC);
  var dropdown_cond = block.getFieldValue('COND');
  var value_var2 = Blockly.Luge.valueToCode(block, 'VAR2', Blockly.Luge.ORDER_ATOMIC);
  var statements_code = Blockly.Luge.statementToCode(block, 'CODE');
  
  var code = 'TantQue ( ' + value_var1 + ' , ' + dropdown_cond +' , ' + value_var2 + ' )\n'+
  statements_code +
  'FinPour\n';
  return code;
};

Blockly.Luge['bool_constant'] = function(block) {
  var dropdown_bool = block.getFieldValue('BOOL');
  
  var code = dropdown_bool;
  return [code, Blockly.Luge.ORDER_ATOMIC];
};

Blockly.Luge['if_else'] = function(block) {
  var value_var1 = Blockly.Luge.valueToCode(block, 'VAR1', Blockly.Luge.ORDER_ATOMIC);
  var dropdown_cond = block.getFieldValue('COND');
  var value_var2 = Blockly.Luge.valueToCode(block, 'VAR2', Blockly.Luge.ORDER_ATOMIC);
  var statements_if_code = Blockly.Luge.statementToCode(block, 'IF_CODE');
  var statements_else_code = Blockly.Luge.statementToCode(block, 'ELSE_CODE');
  
  var code = 'Si ( ' + value_var1 + ' , ' + dropdown_cond + ' ,  ' + value_var2 + ' )\n'+
  'Alors\n'+
  statements_if_code+
  'Sinon\n'+
  statements_else_code+
  'FinSi\n';
  return code;
};

Blockly.Luge['comparison'] = function(block) {
  var value_var1 = Blockly.Luge.valueToCode(block, 'VAR1', Blockly.Luge.ORDER_ATOMIC);
  var dropdown_cond = block.getFieldValue('COND');
  var value_var2 = Blockly.Luge.valueToCode(block, 'VAR2', Blockly.Luge.ORDER_ATOMIC);
  var statements_if_code = Blockly.Luge.statementToCode(block, 'IF_CODE');
  var statements_else_code = Blockly.Luge.statementToCode(block, 'ELSE_CODE');
  
  var code = 'Test ( ' + value_var1 + ' , ' + dropdown_cond + ' ,  ' + value_var2 + ' )\n';
  
  return [code, Blockly.Luge.ORDER_ATOMIC ];
};

Blockly.Luge['acquisition_ana'] = function(block) {
  var value_channel = Blockly.Luge.valueToCode(block, 'CHANNEL', Blockly.Luge.ORDER_ATOMIC);
  var code = 'AcquisitionAnalogique ( ' + value_channel + ' )';
  return [code, Blockly.Luge.ORDER_ATOMIC ];
};

Blockly.Luge['acquisition_tor'] = function(block) {
  var value_channel = Blockly.Luge.valueToCode(block, 'CHANNEL', Blockly.Luge.ORDER_ATOMIC);
  var code = 'AcquisitionTor ( ' + value_channel + ' )';
  return [code, Blockly.Luge.ORDER_ATOMIC ];
};

Blockly.Luge['generation_ana'] = function(block) {
  var value_channel = Blockly.Luge.valueToCode(block, 'CHANNEL', Blockly.Luge.ORDER_ATOMIC);
  var value_value = Blockly.Luge.valueToCode(block, 'VALUE', Blockly.Luge.ORDER_ATOMIC);
  var code = 'GenerationAnalogique ( ' + value_channel + ' , ' + value_value + ' )\n';
  return code;
};

Blockly.Luge['generation_tor'] = function(block) {
  var value_channel = Blockly.Luge.valueToCode(block, 'CHANNEL', Blockly.Luge.ORDER_ATOMIC);
  var value_value = Blockly.Luge.valueToCode(block, 'VALUE', Blockly.Luge.ORDER_ATOMIC);
  var code = 'GenerationTor ( ' + value_channel + ' , ' + value_value + ' )\n';
  return code;
};

