/* Call Library Source File */

// This is a wrapper library to use the libXML2 from GNOME with Labview. 
// See the Readme.txt for details. This file is under LGLP license.
// Author: Markus Lippitz, markus at molphys dot leidenuniv dot nl

// Linux: gcc -fPIC -shared -o lXMLwrap.library.linux lXMLwrap.c -lxml2 -lxslt

// Windows: cl lXMLwrap.c libxml2.lib libxslt.lib /LD


//win
#define PREFIX _declspec(dllexport)


//linux
// #define PREFIX   


#include <libxml/xpath.h>
#include <libxml/xmlschemas.h>
#include <libxml/relaxng.h>
#include <libxslt/transform.h>          


/* ==========================================================================================
   xml Evaluate XPath Expression
   =========================================================================================
 */  
PREFIX  xmlXPathObjectPtr  myxmlXPathEval( xmlDocPtr doc,  char *xpathExpr, xmlNodePtr rootnode)
{
     xmlXPathContextPtr xpathCtx; 
      xmlXPathObjectPtr xpathObj;
     
     
 /* Create xpath evaluation context */
    xpathCtx = xmlXPathNewContext(doc);
    if(xpathCtx != NULL) {
        
           /* set Context node to ROOT node of document, so that relative
              XPath expressions work */
       if (rootnode == NULL)   
              {
                     xpathCtx -> node = doc -> children;
                  }
        else
          {
            xpathCtx -> node = rootnode;
          }  
          
          
            /* Evaluate xpath expression */
           xpathObj = xmlXPathEvalExpression(xpathExpr, xpathCtx);
           xmlXPathFreeContext(xpathCtx); 
           
          /* if(xpathObj == NULL) {
                    xmlXPathFreeContext(xpathCtx); 
                    } */
           }
         else
          {
           xpathObj = NULL;  
           }
           
   return(xpathObj);
}

/* ==========================================================================================
   xml get number of nodes in  XPath
   =========================================================================================
 */  
PREFIX int myxmlgetnodescount( xmlXPathObjectPtr xpathObj)
    {
     int a;
     xmlNodeSetPtr nodes;
     
     nodes = xpathObj->nodesetval;
     if (nodes != NULL) {
             a =  nodes->nodeNr;
            }
         else { a = 0;}   
    return(a);
    }
    

    
/* ==========================================================================================
   xml get root node of a document
   =========================================================================================
 */  
PREFIX xmlNodePtr myxmlgetrootofdoc(  xmlDocPtr doc)
    {
  
    return(doc -> children);
    
    }
    
    
 
 /* ==========================================================================================
   xml  "content" of a "node"
   =========================================================================================
 */  
 
 PREFIX xmlChar *  myxmlgetcontentptr(xmlNodePtr node)
   {
     
     xmlChar *keyword; 
     
     if (node != NULL) {
            keyword = xmlNodeGetContent(node);
          }
        else
         {
           keyword = NULL;   
         }
         
    return(keyword);
    }

  
/* =
 
/* ==========================================================================================
   xml  attribute with name "attribute" of node "node"
   =========================================================================================
 */  
 
 PREFIX xmlChar *  myxmlgetattrvalueptrbyname(xmlNodePtr node, char * attribute)
   {
     
     xmlChar *keyword; 
     
     if (node != NULL) {
            keyword = xmlGetProp(node, attribute);
          }
        else
         {
           keyword = NULL;   
         }
         
    return(keyword);
    }

  
/* ==========================================================================================
   xml "index" node out of XPath node list
   =========================================================================================
 */  

PREFIX  xmlNodePtr  myxmlindexnodelist( xmlXPathObjectPtr xpathObj, int number)
   {
     xmlNodeSetPtr nodes;
     xmlChar *keyword; 
     xmlNodePtr cur;
    

     nodes = xpathObj->nodesetval;
     if (( number >= 0) && (number < nodes->nodeNr)) {
            cur = nodes->nodeTab[number];
            return(cur);
          }
        else
         {
           return(NULL);
         }
   
    }


 

 

    
  /* ==========================================================================================
   xml get the name of a node
   =========================================================================================
 */  

PREFIX xmlChar *  myxmlgetnodestrptr( xmlNodePtr node, int type)
    {
   
     if (node != NULL) {
          if (type == 1) return( (xmlChar *) node -> name  );
      if (type == 2) return ((xmlChar *)  node -> content );
                    } 
                
         return (xmlChar *) NULL;
   
    }
  
    
   /* ==========================================================================================
   xml get elelemnt typeof a node
   =========================================================================================
 */  

PREFIX int  myxmlgetnodeelementtype( xmlNodePtr node)
    {
   
     if (node != NULL) {
          return( node -> type  );
                    } 
                
         return(0);
   
    }
  
 
 /* ==========================================================================================
   xml get the name of a attr
   =========================================================================================
 */  

PREFIX xmlChar *  myxmlgetattrnameptr( xmlAttrPtr node)
    {
  
     if (node != NULL) {
          return( (xmlChar *) node -> name  );
                    } 
                 else 
                   {  return (xmlChar *) NULL ;}
   
    }
    
  /* ==========================================================================================
   xml get some xmlpointer of a node
   =========================================================================================
 */  

PREFIX xmlNodePtr  myxmlnavigatenode( xmlNodePtr node, int type)
    {
    
     if (node != NULL) {
           if (type == 1) return( node -> children  );
       if (type == 2) return( node -> last  );
       if (type == 3) return( node -> parent  );
       if (type == 4) return( node -> next  );
           if (type == 5) return( node -> prev  );
       }
       
    return(NULL);
     }
    
 
     
  /* ==========================================================================================
   xml get attributes pointer
   =========================================================================================
 */  

PREFIX xmlAttrPtr  myxmlnode2attr( xmlNodePtr node)
    {
     
     if (node != NULL) return(node -> properties);
  
    return(NULL);
   
    }
    
      
  /* ==========================================================================================
   xml get some attr_ptr of a attribute
   =========================================================================================
 */  

PREFIX xmlAttrPtr  myxmlnavigateattr( xmlAttrPtr node, int type)
    {
    
     if (node != NULL) {
       if (type == 4) return( node -> next  );
           if (type == 5) return( node -> prev  );
       }
       
    return(NULL);
     }
    
 
     
  /* ==========================================================================================
   xml get attributes pointer
   =========================================================================================
 */  

PREFIX xmlNodePtr  myxmlattr2node( xmlAttrPtr node, int type)
    {
     
     if (node != NULL){
           if (type == 1) return( node -> children  );
       if (type == 3) return( node -> parent  );
       }
    return(NULL);
   
    }
    

    
  /* ==========================================================================================
   xml get xmlChar length
   =========================================================================================
 */  

PREFIX int  myxmlgetlength( xmlChar * string)
    {
     int result;
     
     if (string != NULL) {
                        result = strlen(string);
                         return(result);
                   }
    else {return(-1);}     
           }
           
 /* ==========================================================================================
   xml copy xmlChar to LV
   =========================================================================================
 */  

PREFIX int  myxmlcopystring( xmlChar * string, char * lvdata, int freepointerflag)
    {
        
         strcpy(lvdata,string);  
     if (freepointerflag != 0){ 
            xmlFree(string); }
     return(0);
    }
    
    
/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
       VALIDATE SECTION
  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
*/    

/* 
   The validate-Section is based on 
    XMLStarlet: Command Line Toolkit to query/edit/check/transform XML documents
    Copyright (c) 2002 Mikhail Grushinskiy.  All Rights Reserved.
*/

// **************************************************************************************+
/**
 *  This is the main function for 'validate' option
 */
int valdtd(char * dtdUri, xmlDocPtr doc)
{
            int ret = 0;
        xmlDtdPtr dtd;
        xmlValidCtxtPtr cvp;

            dtd = xmlParseDTD(NULL, (const xmlChar *) dtdUri);
            if (dtd == NULL)
                    ret = -3;  // Could not parse DTD
            else
            {
                 xmlValidCtxtPtr cvp;

                 if ((cvp = xmlNewValidCtxt()) == NULL) 
                    ret = -4; //  Couldn't allocate validation context
                else
            {
                    if (!xmlValidateDtd(cvp, doc, dtd))
                           ret = -5; // file does not match DTD
                      else
                           ret = 0;   // everything OK
                  }  
                 xmlFreeDtd(dtd);
              }
   return ret;
}



// **************************************************************************************+
/**
 *  This is the main function for 'Schema' option
 */
int valSchema(char * schemaUri,  xmlDocPtr doc)
{
    int ret = 0;
    
    xmlSchemaPtr schema = NULL;
    xmlSchemaParserCtxtPtr ctxt = NULL;
    xmlSchemaValidCtxtPtr ctxt2 = NULL;

    ctxt = xmlSchemaNewParserCtxt(schemaUri);
    schema = xmlSchemaParse(ctxt);
    xmlSchemaFreeParserCtxt(ctxt);
    if (schema != NULL)
        {                  
            ctxt2 = xmlSchemaNewValidCtxt(schema);
        if (ctxt2 == NULL)
             ret = -6;
        else
        {    
               ret = xmlSchemaValidateDoc(ctxt2, doc);
               xmlSchemaFreeValidCtxt(ctxt2);
         }  
         xmlSchemaFree(schema);
            
        }
        else    // i.e. schema == NULL
        {
            ret = -7;
        }
       
        xmlSchemaCleanupTypes();        
   return ret;
}



// **************************************************************************************+
/**
 *  This is the main function for 'Relax NG' option
 */
int valRelaxNG(char * relaxNGuri,  xmlDocPtr doc)
{
     int ret = 0;
     xmlRelaxNGPtr relaxngschemas = NULL;
     xmlRelaxNGParserCtxtPtr ctxt = NULL;
     xmlRelaxNGValidCtxtPtr ctxt2 = NULL;
   
     /* forces loading the DTDs */
    xmlLoadExtDtdDefaultValue |= 1;

    ctxt = xmlRelaxNGNewParserCtxt(relaxNGuri);
    relaxngschemas = xmlRelaxNGParse(ctxt);
    xmlRelaxNGFreeParserCtxt(ctxt);     

    if (relaxngschemas != NULL)
      {
          ctxt2 = xmlRelaxNGNewValidCtxt(relaxngschemas);
      if (ctxt2 == NULL)
            ret = -8;
      else
      { 
             ret = xmlRelaxNGValidateDoc(ctxt2, doc);
              xmlRelaxNGFreeValidCtxt(ctxt2);
        }  
           xmlRelaxNGFree(relaxngschemas);
     }
    else
       ret = -9;
     
   xmlRelaxNGCleanupTypes();
 
   return ret;
}





/* ==========================================================================================
   myXMLvalidate: validates XML against several types:
              1 : DTD
          2 : Schema
          3 : RelaxNG
   =========================================================================================
 */ 

PREFIX int myXMLvalidate(int validationType, char * againstUri, xmlDocPtr doc )
{
     int ret = 0;
    
    if (doc)
          {
       
      if (validationType == 1)
                ret = valdtd( againstUri, doc);
      else 
      if (validationType == 2)
                ret = valSchema( againstUri, doc);
    else 
      if (validationType == 3)
                ret = valRelaxNG( againstUri, doc);
    else ret = -1;
    }
  else
      ret = -2;
      
    return ret;  
    }               



/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
       TRANSLATE SECTION
  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
*/      

 /* based on
 * libxslt_tutorial.c: demo program for the XSL Transformation 1.0 engine
 * based on xsltproc.c, by Daniel.Veillard@imag.fr
 * by John Fleck 
 *
 */ 


// was in the "tutorial" file, but apparently not needed
// extern int xmlLoadExtDtdDefaultValue;

/* ==========================================================================================
   myXMLtranslate: very simple translate function: XML --XSL--> outFile
   =========================================================================================
 */ 
       
PREFIX int myXMLtranslate(char * xslFile, char * xmlFile, char * outFile) {
       
        int ret;
        const char *params[16 + 1];
        xsltStylesheetPtr cur = NULL;
        xmlDocPtr doc, res;

        // here we could write some parameters in, but might be difficult from LV, so omitted
        params[0] = NULL;
        xmlSubstituteEntitiesDefault(1);
        xmlLoadExtDtdDefaultValue = 1;
        cur = xsltParseStylesheetFile((const xmlChar *) xslFile);
       
    if (cur == NULL)
               ret = -1;    // unable to parse xsl
    else
    {      
            doc = xmlParseFile(xmlFile);
        if (doc == NULL)
               ret = -2;     // unable to parse xml
        else
        {      
                 res = xsltApplyStylesheet(cur, doc, params);
         if (res == NULL)
                 ret = -3;   // error while translating
        else
        {    
                        ret = xsltSaveResultToFilename(outFile, res, cur,0);  // # of bytes written
                         xmlFreeDoc(res);
        }   
         xmlFreeDoc(doc);
         }    
             xsltFreeStylesheet(cur);
     }

        xsltCleanupGlobals();
        xmlCleanupParser();
        return ret;
}
